#  Copyright © 2025 Bentley Systems, Incorporated
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#      http://www.apache.org/licenses/LICENSE-2.0
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from __future__ import annotations

__all__ = [
    "FileMetadata",
    "FileVersion",
]

from dataclasses import dataclass
from datetime import datetime

from evo.common import ResourceMetadata, ServiceUser


@dataclass(frozen=True, kw_only=True)
class FileMetadata(ResourceMetadata):
    """Metadata about a file in the File API."""

    parent: str
    """The parent path of the file."""

    version_id: str
    """An arbitrary identifier for the file version."""

    size: int
    """The size of the file in bytes."""

    modified_at: datetime
    """The resource's last modified timestamp."""

    modified_by: ServiceUser | None = None
    """The user who last modified the resource."""

    @property
    def path(self) -> str:
        """The full path of the file, formed by joining the parent and name, separated by a slash ('/')."""
        return f"{self.parent.removesuffix('/')}/{self.name.removeprefix('/')}"

    @property
    def url(self) -> str:
        """The URL of the file in the File API."""
        return (
            "{hub_url}/file/v2/orgs/{org_id}/workspaces/{workspace_id}/files/{file_id}?version_id={version_id}".format(
                hub_url=self.environment.hub_url.rstrip("/"),
                org_id=self.environment.org_id,
                workspace_id=self.environment.workspace_id,
                file_id=self.id,
                version_id=self.version_id,
            )
        )


@dataclass(frozen=True, kw_only=True)
class FileVersion:
    """Represents a version of a file."""

    version_id: str
    """An arbitrary identifier for the file version."""

    created_at: datetime
    """The date and time when the file version was created."""

    created_by: ServiceUser | None
    """The user who uploaded the file version."""



#  Copyright © 2025 Bentley Systems, Incorporated
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#      http://www.apache.org/licenses/LICENSE-2.0
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
"""
File API
=============

The File API provides the ability to manage files of any type or size, associated with
your Evo workspace. Enable your product with Evo connected workflows by integrating with the Seequent Evo
File API. Most file formats and sizes are accepted.

Files can be referenced by their UUID, or by a user-defined file path. Files are versioned, so updating or
replacing them will create a new version of the file. The latest version of the file is always returned
unless a specific version is requested.

For more information on using the File API, see [Overview](https://developer.seequent.com/docs/guides/file/), or the API references here.


This code is generated from the OpenAPI specification for File API.
API version: 2.8.0
"""

# Import endpoint apis.
from .api import FileV2Api

__all__ = [
    "FileV2Api",
]



#  Copyright © 2025 Bentley Systems, Incorporated
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#      http://www.apache.org/licenses/LICENSE-2.0
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""FileAPI SDK
=====================
"""

from .client import FileAPIClient
from .data import FileMetadata, FileVersion
from .io import FileAPIDownload, FileAPIUpload

__all__ = [
    "FileAPIClient",
    "FileAPIDownload",
    "FileAPIUpload",
    "FileMetadata",
    "FileVersion",
]



#  Copyright © 2025 Bentley Systems, Incorporated
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#      http://www.apache.org/licenses/LICENSE-2.0
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

# generated by datamodel-codegen:
#   filename:  spec.yaml

from __future__ import annotations

from datetime import datetime
from typing import Annotated
from uuid import UUID

from pydantic import Field, StrictInt, StrictStr

from .._model_config import CustomBaseModel


class ListFileLinks(CustomBaseModel):
    self: Annotated[StrictStr, Field(title="Self")]


class ListFilesResponseLinks(CustomBaseModel):
    next: Annotated[StrictStr | None, Field(title="Next")] = None
    prev: Annotated[StrictStr | None, Field(title="Prev")] = None
    self: Annotated[StrictStr, Field(title="Self")]


class UploadFileResponse(CustomBaseModel):
    """
    Upload file response
    """

    file_id: Annotated[UUID, Field(title="File Id")]
    upload: Annotated[StrictStr, Field(title="Upload")]
    version_id: Annotated[StrictStr, Field(title="Version Id")]


class UserInfo(CustomBaseModel):
    email: Annotated[StrictStr | None, Field(examples=["kim@example.test"], title="Email Address")] = None
    """
    The primary email address of the user. Can be null if an error occurred while retrieving this information.
    """
    id: Annotated[UUID, Field(examples=["59b73891-5538-4e45-ae67-f8c5b00d7405"], title="User ID")]
    """
    The ID of the user
    """
    name: Annotated[StrictStr | None, Field(examples=["Kim Kim"], title="Full Name")] = None
    """
    The full name of the user. Can be null if an error occurred while retrieving this information.
    """


class FileVersionResponse(CustomBaseModel):
    """
    Metadata for specific file version
    """

    created_at: Annotated[datetime, Field(title="Created At")]
    created_by: UserInfo | None = None
    file_id: Annotated[UUID, Field(title="File Id")]
    link: Annotated[StrictStr, Field(title="Link")]
    name: Annotated[StrictStr, Field(title="Name")]
    path: Annotated[StrictStr, Field(title="Path")]
    size: Annotated[StrictInt, Field(title="Size")]
    version_id: Annotated[StrictStr, Field(title="Version Id")]


class ListFile(CustomBaseModel):
    created_at: Annotated[datetime, Field(title="Created At")]
    created_by: UserInfo | None = None
    deleted_at: Annotated[datetime | None, Field(title="Deleted At")] = None
    deleted_by: UserInfo | None = None
    etag: Annotated[StrictStr, Field(title="Etag")]
    file_id: Annotated[UUID, Field(title="File Id")]
    links: ListFileLinks
    modified_at: Annotated[datetime, Field(title="Modified At")]
    modified_by: UserInfo | None = None
    name: Annotated[StrictStr, Field(title="Name")]
    path: Annotated[StrictStr, Field(title="Path")]
    size: Annotated[StrictInt, Field(title="Size")]
    version_id: Annotated[StrictStr, Field(title="Version Id")]


class ListFilesResponse(CustomBaseModel):
    """
    List files.
    """

    count: Annotated[StrictInt, Field(title="Count")]
    files: Annotated[list[ListFile], Field(title="Files")]
    limit: Annotated[StrictInt, Field(title="Limit")]
    links: ListFilesResponseLinks
    offset: Annotated[StrictInt, Field(title="Offset")]
    total: Annotated[StrictInt, Field(title="Total")]


class DownloadFileResponse(CustomBaseModel):
    """
    Download file response.
    """

    created_at: Annotated[datetime, Field(title="Created At")]
    created_by: UserInfo | None = None
    deleted_at: Annotated[datetime | None, Field(title="Deleted At")] = None
    deleted_by: UserInfo | None = None
    download: Annotated[StrictStr, Field(title="Download")]
    etag: Annotated[StrictStr, Field(title="Etag")]
    file_id: Annotated[UUID, Field(title="File Id")]
    modified_at: Annotated[datetime, Field(title="Modified At")]
    modified_by: UserInfo | None = None
    name: Annotated[StrictStr, Field(title="Name")]
    path: Annotated[StrictStr, Field(title="Path")]
    self: Annotated[StrictStr, Field(title="Self")]
    size: Annotated[StrictInt, Field(title="Size")]
    version_id: Annotated[StrictStr, Field(title="Version Id")]
    versions: Annotated[list[FileVersionResponse] | None, Field(title="Versions")] = None
